@@ -29,9 +29,7 @@
 #define __SYNFIG_STUDIO_APP_H
 
 /* === H E A D E R S ======================================================= */
-
-#include <sigc++/bind.h>
-
+#include <sigc++/sigc++.h>
 #include <gtkmm/main.h>
 #include <string>
 #include <list>
@@ -49,6 +47,7 @@
 #include <synfigapp/instance.h>
 #include <synfigapp/pluginmanager.h>
 #include "iconcontroller.h"
+#include "mainwindow.h"
 
 /* === M A C R O S ========================================================= */
 
@@ -59,7 +58,7 @@
 #define RENDER_DIR_PREFERENCE		"render_dir"
 
 // uncomment define SINGLE_THREADED to use a single thread, and hopefully get more stability (dooglus) - changed to be default if using windows (Nov 2009 pixelgeek)
-#ifdef WIN32
+#ifdef _WIN32
 #define SINGLE_THREADED
 #endif
 
@@ -209,7 +208,7 @@ public:
 	static bool shutdown_in_progress;
 
 	static bool use_colorspace_gamma;
-	
+
 #ifdef SINGLE_THREADED
 	static bool single_threaded;
 #endif
@@ -218,11 +217,13 @@ public:
 	static bool resize_imported_images;
 	static bool enable_experimental_features;
 	static bool use_dark_theme;
+	static bool show_file_toolbar;
 
 	static synfigapp::PluginManager plugin_manager;
-	
+
 	static synfig::String browser_command;
-	static synfig::String brushes_path;
+//	static synfig::String brushes_path;
+	static std::set< synfig::String > brushes_path;
 	static synfig::String custom_filename_prefix;
 	static int preferred_x_size;
 	static int preferred_y_size;
@@ -230,10 +231,11 @@ public:
 	static synfig::String predefined_fps;
 	static float preferred_fps;
 	static synfig::String sequence_separator;
-	static bool navigator_uses_cairo;
-	static bool workarea_uses_cairo;
+	static synfig::String navigator_renderer;
+	static synfig::String workarea_renderer;
 	static bool enable_mainwin_menubar;
 	static synfig::String ui_language;
+	static long ui_handle_tooltip_flag;
 
 	/*
  -- ** -- S I G N A L S -------------------------------------------------------
@@ -376,6 +378,7 @@ public:
 	static bool dialog_open_file_image(const std::string &title, std::string &filename, std::string preference);
 	static bool dialog_open_file_audio(const std::string &title, std::string &filename, std::string preference);
 	static bool dialog_open_file_with_history_button(const std::string &title, std::string &filename, bool &show_history, std::string preference);
+	static bool dialog_open_folder(const std::string &title, std::string &filename, std::string preference, Gtk::Window& transientwind=*App::main_window);
 	static bool dialog_save_file(const std::string &title, std::string &filename, std::string preference);
 	static bool dialog_save_file_spal(const std::string &title, std::string &filename, std::string preference);
 	static bool dialog_save_file_sketch(const std::string &title, std::string &filename, std::string preference);
@@ -411,7 +414,7 @@ public:
 			const std::string &button2,
 			const std::string &button3);
 
-	static void open_url(const std::string &url);
+	static void open_uri(const std::string &uri);
 
 	static synfig::String get_config_file(const synfig::String& file);
 	// This will spread the changes made in preferences.
