@@ -45,7 +45,7 @@
 #include "about.h"
 #include "app.h"
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -62,7 +62,7 @@ using namespace studio;
 #define PACKAGE	"synfigstudio"
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
 #	ifdef IMAGE_DIR
 #		undef IMAGE_DIR
 #		define IMAGE_DIR "share\\pixmaps"
@@ -188,7 +188,7 @@ About::About()
 	set_translator_credits(_("translator-credits"));
 
 	std::string imagepath;
-#ifdef WIN32
+#ifdef _WIN32
 	imagepath=App::get_base_path()+ETL_DIRECTORY_SEPARATOR+IMAGE_DIR;
 #else
 	imagepath=IMAGE_DIR;
@@ -204,7 +204,7 @@ About::About()
 	imagepath+=ETL_DIRECTORY_SEPARATOR;
 
 	Gtk::Image *Logo = manage(new class Gtk::Image());
-	Logo->set(imagepath+"synfig_icon."IMAGE_EXT);
+	Logo->set(imagepath+"synfig_icon." IMAGE_EXT);
 	set_logo(Logo->get_pixbuf());
 
 #ifdef SHOW_EXTRA_INFO
@@ -254,5 +254,5 @@ void About::close(int){
 
 void About::on_link_clicked(Gtk::AboutDialog&, const Glib::ustring &url)
 {
-	App::open_url(url);
+	App::open_uri(url);
 }
