@@ -30,6 +30,8 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+
 #include "docks/dock_info.h"
 #include "canvasview.h"
 #include "workarea.h"
@@ -41,7 +43,7 @@
 #include <gtkmm/separator.h>
 #include <gtkmm/invisible.h>
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -63,13 +65,15 @@ using namespace synfig;
 
 void studio::Dock_Info::on_mouse_move()
 {
-	Point pos = get_canvas_view()->work_area->get_cursor_pos();
+	etl::loose_handle<CanvasView> canvas_view(get_canvas_view());
+	if(!canvas_view) return;
+	Point pos = canvas_view->work_area->get_cursor_pos();
 
 	Distance xv(pos[0],Distance::SYSTEM_UNITS);
-	xv.convert(App::distance_system, get_canvas_view()->get_canvas()->rend_desc());
+	xv.convert(App::distance_system, canvas_view->get_canvas()->rend_desc());
 
 	Distance yv(pos[1],Distance::SYSTEM_UNITS);
-	yv.convert(App::distance_system, get_canvas_view()->get_canvas()->rend_desc());
+	yv.convert(App::distance_system, canvas_view->get_canvas()->rend_desc());
 
 	//get the color and set the labels
 
@@ -77,18 +81,8 @@ void studio::Dock_Info::on_mouse_move()
 	y.set_text(yv.get_string(3));
 
 	float cr=0.f,cg=0.f,cb=0.f,ca=0.f;
-	if (App::workarea_uses_cairo)
-	{
-		// TODO: This is disabled for now, because it crashes Synfig when *.lst files are imported
-		
-		//CairoColor c = get_canvas_view()->get_canvas()->get_context( get_canvas_view()->get_context_params() ).get_cairocolor(pos);
-		//cr = c.get_r(); cg = c.get_g(); cb = c.get_b(); ca = c.get_a();
-	}
-	else
-	{
-		Color c = get_canvas_view()->get_canvas()->get_context( get_canvas_view()->get_context_params() ).get_color(pos);
-		cr = c.get_r(); cg = c.get_g(); cb = c.get_b(); ca = c.get_a();
-	}
+	Color c = canvas_view->get_canvas()->get_context( canvas_view->get_context_params() ).get_color(pos);
+	cr = c.get_r(); cg = c.get_g(); cb = c.get_b(); ca = c.get_a();
 
 	if(use_colorspace_gamma())
 	{
