@@ -33,9 +33,13 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+
 #include "instance.h"
 #include <cassert>
 
+#include <giomm.h>
+
 #include <gtkmm/stock.h>
 #include <gtkmm/image.h>
 #include <gtkmm/menuitem.h>
@@ -46,18 +50,17 @@
 #include <iostream>
 #include "canvasview.h"
 #include "app.h"
-#include <sigc++/signal.h>
-#include <sigc++/adaptors/hide.h>
+#include <sigc++/sigc++.h>
 #include "docks/dock_toolbox.h"
 #include "onemoment.h"
 #include <synfig/savecanvas.h>
+#include <synfig/canvasfilenaming.h>
+#include <synfig/layers/layer_pastecanvas.h>
 
 #include "autorecover.h"
-#include <sigc++/retype_return.h>
-#include <sigc++/retype.h>
-//#include <sigc++/hide.h>
 #include <synfig/valuenodes/valuenode_composite.h>
 #include <synfig/valuenodes/valuenode_duplicate.h>
+#include <synfig/widthpoint.h>
 #include "widgets/widget_waypointmodel.h"
 #include <gtkmm/actiongroup.h>
 #include "iconcontroller.h"
@@ -66,7 +69,7 @@
 #include <errno.h>
 #include <ETL/stringf>
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -86,7 +89,7 @@ int studio::Instance::instance_count_=0;
 
 /* === M E T H O D S ======================================================= */
 
-Instance::Instance(synfig::Canvas::Handle canvas, etl::handle< synfig::FileContainerTemporary > container):
+Instance::Instance(synfig::Canvas::Handle canvas, synfig::FileSystemTemporary::Handle container):
 	synfigapp::Instance		(canvas, container),
 	canvas_tree_store_		(Gtk::TreeStore::create(canvas_tree_model)),
 	history_tree_store_		(HistoryTreeStore::create(this)),
@@ -128,7 +131,7 @@ Instance::get_visible_canvases()const
 }
 
 handle<Instance>
-Instance::create(synfig::Canvas::Handle canvas, etl::handle< synfig::FileContainerTemporary > container)
+Instance::create(synfig::Canvas::Handle canvas, synfig::FileSystemTemporary::Handle container)
 {
 	// Construct a new instance
 	handle<Instance> instance(new Instance(canvas, container));
@@ -211,13 +214,13 @@ studio::Instance::run_plugin(std::string plugin_path)
 				synfigapp::UIInterface::RESPONSE_OK);
 
 	if(answer == synfigapp::UIInterface::RESPONSE_OK){
-	
+
 		OneMoment one_moment;
 
 		Canvas::Handle canvas(this->get_canvas());
-		
+
 		synfigapp::PluginLauncher launcher(canvas);
-		
+
 		Time cur_time;
 		cur_time = canvas->get_time();
 
@@ -250,16 +253,16 @@ studio::Instance::run_plugin(std::string plugin_path)
 						_("Close"));
 
 				one_moment.show();
-				
+
 			}
 		}
-	
-	
+
+
 		canvas=0;
 
 		App::open_as(launcher.get_result_path(),launcher.get_original_path());
-	
-	
+
+
 		etl::handle<Instance> new_instance = App::instance_list.back();
 		new_instance->inc_action_count(); // This file isn't saved! mark it as such
 
@@ -527,8 +530,10 @@ Instance::close()
 		studio::App::set_selected_canvas_view(0);
 		studio::App::set_selected_instance(0);
 	}
-	else
+	else if(studio::App::instance_list.size() == 1)
+	{
 		studio::App::instance_list.front()->canvas_view_list().front()->present();
+	}
 }
 
 void
@@ -1217,7 +1222,7 @@ Instance::make_param_menu(Gtk::Menu *menu,synfig::Canvas::Handle canvas, synfiga
 		item->show();
 		parammenu.append(*item);
 	}
-	
+
 	// Interpolation menu: Show it only if
 	// the value description is constant or animated (layer parameter or exported) and not static
 	// and value is not canvas type
@@ -1259,14 +1264,14 @@ Instance::make_param_menu(Gtk::Menu *menu,synfig::Canvas::Handle canvas, synfiga
 		item->show_all(); \
 		param_interpolation_menu->append(*item); \
 		param_list.erase("new_value");
-		
-		
+
+
 		ADD_IMAGE_MENU_ITEM(INTERPOLATION_TCB, "synfig-interpolation_type_tcb", _("TCB"));
 		ADD_IMAGE_MENU_ITEM(INTERPOLATION_LINEAR, "synfig-interpolation_type_linear", _("Linear"));
 		ADD_IMAGE_MENU_ITEM(INTERPOLATION_HALT, "synfig-interpolation_type_ease", _("Ease"));
 		ADD_IMAGE_MENU_ITEM(INTERPOLATION_CONSTANT, "synfig-interpolation_type_const", _("Constant"));
 		ADD_IMAGE_MENU_ITEM(INTERPOLATION_CLAMPED, "synfig-interpolation_type_clamped", _("Clamped"));
-		
+
 		#undef ADD_IMAGE_MENU_ITEM
 
 		item = Gtk::manage(new Gtk::MenuItem(_("Interpolation")));
@@ -1369,7 +1374,7 @@ Instance::make_param_menu(Gtk::Menu *menu,synfig::Canvas::Handle canvas, synfiga
 		item = Gtk::manage(new Gtk::SeparatorMenuItem());
 		item->show();
 		parammenu.append(*item);
-		
+
 		////// After ///////////////////////
 		param_list.erase("value_desc");
 		param_list.add("value_desc",synfigapp::ValueDesc(wpoint_composite, wpoint_composite->get_link_index_from_name("side_after")));
@@ -1571,3 +1576,159 @@ Instance::make_param_menu(Gtk::Menu *menu,synfig::Canvas::Handle canvas,const st
 	if (value_desc.is_valid())
 		make_param_menu(menu,canvas,value_desc, 0.f, false);
 }
+
+
+void
+Instance::gather_uri(std::set<synfig::String> &x, const synfig::ValueNode::Handle &value_node) const
+{
+	if (!value_node || !value_node->get_parent_canvas()) return;
+
+	LinkableValueNode::Handle linkable_value_node = LinkableValueNode::Handle::cast_dynamic(value_node);
+	if (!linkable_value_node) return;
+
+	FileSystem::Handle file_system = value_node->get_parent_canvas()->get_file_system();
+	if (!file_system) return;
+
+	Time t = value_node->get_parent_canvas()->get_time();
+
+	ParamVocab vocab = linkable_value_node->get_children_vocab();
+	for(ParamVocab::const_iterator i = vocab.begin(); i != vocab.end(); ++i)
+	{
+		ValueNode::Handle child_node = linkable_value_node->get_link(i->get_name());
+		if (!child_node) continue;
+
+		if (i->get_hint() == "filename")
+		{
+			ValueBase v = (*child_node)(t);
+			if (v.can_get(String()))
+			{
+				String filename = CanvasFileNaming::make_full_filename(value_node->get_parent_canvas()->get_file_name(), v.get(String()));
+				String uri = file_system->get_real_uri(filename);
+				if (!uri.empty()) x.insert(uri);
+			}
+		}
+
+		gather_uri(x, child_node);
+	}
+}
+
+void
+Instance::gather_uri(std::set<synfig::String> &x, const synfig::Layer::Handle &layer) const
+{
+	if (!layer || !layer->get_canvas()) return;
+
+	FileSystem::Handle file_system = layer->get_canvas()->get_file_system();
+	if (!file_system) return;
+
+	ParamVocab vocab = layer->get_param_vocab();
+	for(ParamVocab::const_iterator i = vocab.begin(); i != vocab.end(); ++i)
+	{
+		if (i->get_hint() == "filename")
+		{
+			ValueBase v = layer->get_param(i->get_name());
+			if (v.can_get(String()))
+			{
+				String filename = CanvasFileNaming::make_full_filename(layer->get_canvas()->get_file_name(), v.get(String()));
+				String uri = file_system->get_real_uri(filename);
+				if (!uri.empty()) x.insert(uri);
+			}
+		}
+
+		Layer::DynamicParamList::const_iterator j = layer->dynamic_param_list().find(i->get_name());
+		if (j != layer->dynamic_param_list().end() && j->second)
+			gather_uri(x, j->second);
+	}
+
+	//if (etl::handle<Layer_PasteCanvas> paste = etl::handle<Layer_PasteCanvas>::cast_dynamic(layer))
+	//	gather_uri(x, paste->get_param("canvas").get(Canvas::Handle()));
+}
+
+void
+Instance::gather_uri(std::set<synfig::String> &x, const synfig::Canvas::Handle &canvas) const
+{
+	if (!canvas) return;
+
+	FileSystem::Handle file_system = canvas->get_file_system();
+	if (!file_system) return;
+
+	for(Canvas::const_iterator i = canvas->begin(); i != canvas->end(); ++i)
+		gather_uri(x, *i);
+}
+
+void
+Instance::gather_uri(std::set<synfig::String> &x, const synfigapp::SelectionManager::LayerList &layers) const
+{
+	for(synfigapp::SelectionManager::LayerList::const_iterator i = layers.begin(); i != layers.end(); ++i)
+		gather_uri(x, *i);
+}
+
+void
+Instance::gather_uri(std::map<synfig::String, synfig::String> &x, const synfigapp::SelectionManager::LayerList &layers) const
+{
+	std::set<String> uri_set;
+	gather_uri(uri_set, layers);
+	std::vector<String> uri_list;
+	std::vector<String> filename_list;
+	for(std::set<String>::const_iterator i = uri_set.begin(); i != uri_set.end(); ++i)
+	{
+		String filename = Glib::filename_from_uri(*i);
+		if (!filename.empty())
+		{
+			uri_list.push_back(*i);
+			filename_list.push_back(filename);
+		}
+	}
+
+	std::vector<String> shortnames;
+	MainWindow::make_short_filenames(filename_list, shortnames);
+	for(int i = 0; i < (int)uri_list.size() && i < (int)shortnames.size(); ++i)
+		x[shortnames[i]] = uri_list[i];
+}
+
+void
+Instance::add_special_layer_actions_to_menu(Gtk::Menu *menu, const synfig::Layer::Handle &layer) const
+{
+	add_special_layer_actions_to_menu(menu, synfigapp::SelectionManager::LayerList(1, layer));
+}
+
+void
+Instance::add_special_layer_actions_to_group(const Glib::RefPtr<Gtk::ActionGroup>& action_group, synfig::String& ui_info, const synfig::Layer::Handle &layer) const
+{
+	add_special_layer_actions_to_group(action_group, ui_info, synfigapp::SelectionManager::LayerList(1, layer));
+}
+
+void
+Instance::add_special_layer_actions_to_menu(Gtk::Menu *menu, const synfigapp::SelectionManager::LayerList &layers) const
+{
+	std::map<String, String> uris;
+	gather_uri(uris, layers);
+	for(std::map<String, String>::const_iterator i = uris.begin(); i != uris.end(); ++i)
+	{
+		Gtk::MenuItem *item = manage(new Gtk::ImageMenuItem(Gtk::Stock::OPEN));
+		item->set_label( (String(_("Open file")) + " '" + i->first + "'").c_str() );
+		item->signal_activate().connect(
+			sigc::bind(sigc::ptr_fun(&App::open_uri), i->second) );
+		item->show();
+		menu->append(*item);
+	}
+}
+
+void
+Instance::add_special_layer_actions_to_group(const Glib::RefPtr<Gtk::ActionGroup>& action_group, synfig::String& ui_info, const synfigapp::SelectionManager::LayerList &layers) const
+{
+	std::map<String, String> uris;
+	gather_uri(uris, layers);
+	int index = 0;
+	for(std::map<String, String>::const_iterator i = uris.begin(); i != uris.end(); ++i, ++index)
+	{
+		String action_name = etl::strprintf("special-action-open-file-%d", index);
+		String local_name = String(_("Open file")) + " '" + i->first + "'";
+		action_group->add(
+			Gtk::Action::create(
+				action_name,
+				Gtk::Stock::OPEN,
+				local_name, local_name ),
+			sigc::bind(sigc::ptr_fun(&App::open_uri), i->second) );
+		ui_info += strprintf("<menuitem action='%s' />", action_name.c_str());
+	}
+}
