@@ -32,6 +32,8 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+
 #include "preview.h"
 #include "app.h"
 #include "audiocontainer.h"
@@ -46,14 +48,14 @@
 #include <algorithm>
 #include "asyncrenderer.h"
 
-#include "general.h"
+#include <gui/localization.h>
 
 #include <cmath>
 #include <cassert>
 #include <algorithm>
 #include <cstdio>
 #include <ctype.h>
-#include <math.h>
+#include <cmath>
 #include <synfig/string.h>
 
 #endif
