@@ -33,6 +33,8 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+
 #include <iostream>
 #include <string>
 
@@ -42,13 +44,12 @@
 #include <gtkmm/label.h>
 #include <gtkmm/frame.h>
 #include <gtkmm/fixed.h>
-
-#include <synfig/general.h>
+#include <gdkmm/rgba.h>
 
 #include "splash.h"
 #include "app.h"
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -63,7 +64,7 @@ using namespace studio;
 #define PACKAGE	"synfigstudio"
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
 #	ifdef IMAGE_DIR
 #		undef IMAGE_DIR
 #		define IMAGE_DIR "share\\pixmaps"
@@ -99,10 +100,10 @@ public:
 		}
 		else
 		{
-			cerr<<task.c_str()<<endl;
+			synfig::info(task);
 		}
 
-		while(studio::App::events_pending())studio::App::iteration(false);
+		while(studio::App::events_pending()) studio::App::iteration(false);
 		return true;
 	}
 
@@ -113,12 +114,10 @@ public:
 			splash.tasklabel->set_label(_("ERROR:")+task);
 			splash.tasklabel->show();
 		}
-		else
-		{
-			cerr<<task.c_str()<<endl;
-		}
 
-		while(studio::App::events_pending())studio::App::iteration(false);
+		synfig::error(task);
+
+		while(studio::App::events_pending()) studio::App::iteration(false);
 		return true;
 	}
 
@@ -129,12 +128,10 @@ public:
 			splash.tasklabel->set_label(_("WARNING:")+task);
 			splash.tasklabel->show();
 		}
-		else
-		{
-			cerr<<task.c_str()<<endl;
-		}
 
-		while(studio::App::events_pending())studio::App::iteration(false);
+		synfig::warning(task);
+
+		while(studio::App::events_pending()) studio::App::iteration(false);
 		return true;
 	}
 
@@ -159,7 +156,7 @@ Splash::Splash():
 	Gtk::Window(getenv("SYNFIG_DISABLE_POPUP_WINDOWS") ? Gtk::WINDOW_TOPLEVEL : Gtk::WINDOW_POPUP)
 {
 	std::string imagepath;
-#ifdef WIN32
+#ifdef _WIN32
 	imagepath=App::get_base_path()+ETL_DIRECTORY_SEPARATOR+IMAGE_DIR;
 #else
 	imagepath=IMAGE_DIR;
@@ -185,9 +182,9 @@ Splash::Splash():
 	if(ran >0.499999)
 		number = 2;
 	//synfig::info("%s", strprintf("%d",number).c_str());
-	splash_image->set(imagepath+"splash_screen"+strprintf("%d",number)+"."IMAGE_EXT);
+	splash_image->set(imagepath+"splash_screen"+strprintf("%d",number)+"." IMAGE_EXT);
 	*/
-	splash_image->set(imagepath+"splash_screen."IMAGE_EXT);
+	splash_image->set(imagepath+"splash_screen." IMAGE_EXT);
 	splash_image->set_alignment(0.5,0.5);
 	splash_image->set_padding(0,0);
 
@@ -208,11 +205,20 @@ Splash::Splash():
 	tasklabel->set_size_request(image_w,24);
 	tasklabel->set_use_underline(false);
 
+	// Create the current task label
+	versionlabel = manage(new class Gtk::Label());
+	versionlabel->set_label("" VERSION);
+	versionlabel->set_size_request(image_w,24);
+	versionlabel->set_use_underline(false);
+	versionlabel->override_color(Gdk::RGBA("#FFFFFF"));
+	versionlabel->show();
+
 	// Create the Gtk::Fixed container and put all of the widgets into it
 	Gtk::Fixed* fixed = manage(new class Gtk::Fixed());
 	if( pixbuf ) fixed->put(*splash_image, 0, 0);
 	fixed->put(*progressbar, 0, image_h+24);
 	fixed->put(*tasklabel, 0, image_h);
+	fixed->put(*versionlabel, 0, image_h-24);
 
 	// Create shadow around the outside of the window
 	Gtk::Frame* frame = manage(new class Gtk::Frame());
@@ -220,7 +226,7 @@ Splash::Splash():
   	frame->add(*fixed);
 
 	// Set up the parameters for this pop-up window
-	set_title("Synfig Studio "VERSION);
+	set_title("Synfig Studio " VERSION);
 	set_modal(false);
 	property_window_position().set_value(Gtk::WIN_POS_CENTER);
 	set_resizable(false);
