@@ -1,6 +1,6 @@
 /* === S Y N F I G ========================================================= */
 /*!	\file workarea.cpp
-**	\brief Template Header
+**	\brief Work area
 **
 **	$Id$
 **
@@ -8,7 +8,8 @@
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2006 Yue Shi Lai
 **	Copyright (c) 2007, 2008 Chris Moore
-**  Copyright (c) 2011 Nikita Kitaev
+**	Copyright (c) 2011 Nikita Kitaev
+**	Copyright (c) 2016 caryoscelus
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -32,28 +33,25 @@
 #	include <config.h>
 #endif
 
-#include <sigc++/adaptors/hide.h>
+#include <synfig/general.h>
 
 #include "workarea.h"
 #include "canvasview.h"
-#include "app.h"
-#include <gtkmm/window.h>
-#include <gtkmm/image.h>
-#include <gtkmm/drawingarea.h>
 #include <gtkmm/arrow.h>
-#include <gtkmm/image.h>
+#include <gtkmm/frame.h>
 #include <gtkmm/scrollbar.h>
+#include <gtkmm/window.h>
+
 #include <cmath>
-#include <sigc++/retype_return.h>
-#include <sigc++/retype.h>
-#include <sigc++/hide.h>
 #include <ETL/misc>
 
+#include <synfig/debug/debugsurface.h>
 #include <synfig/target_scanline.h>
 #include <synfig/target_tile.h>
 #include <synfig/target_cairo.h>
 #include <synfig/target_cairo_tile.h>
 #include <synfig/surface.h>
+#include <synfig/blinepoint.h>
 #include <synfig/valuenodes/valuenode_composite.h>
 #include <synfigapp/canvasinterface.h>
 #include "event_mouse.h"
@@ -74,11 +72,10 @@
 #include "workarearenderer/renderer_dragbox.h"
 #include "workarearenderer/renderer_bbox.h"
 #include "asyncrenderer.h"
-#include <gtkmm/frame.h>
 
 #include <synfig/mutex.h>
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -101,223 +98,6 @@ using namespace studio;
 
 /* === C L A S S E S ======================================================= */
 
-class studio::WorkAreaTarget_Cairo_Tile : public synfig::Target_Cairo_Tile
-{
-public:
-	WorkArea *workarea;
-	bool low_res;
-	int w,h;
-	int real_tile_w,real_tile_h;
-	int refresh_id;
-	
-	bool onionskin;
-	bool onion_first_tile;
-	int onion_layers;
-	
-	std::list<synfig::Time> onion_skin_queue;
-	
-	synfig::Mutex mutex;
-	
-	void set_onion_skin(bool x, int *onions)
-	{
-		onionskin=x;
-		
-		Time time(rend_desc().get_time_start());
-		
-		if(!onionskin)
-			return;
-		onion_skin_queue.push_back(time);
-		
-		try
-		{
-			Time thistime=time;
-			for(int i=0; i<onions[0]; i++)
-			{
-				Time keytime=get_canvas()->keyframe_list().find_prev(thistime)->get_time();
-				onion_skin_queue.push_back(keytime);
-				thistime=keytime;
-			}
-		}
-		catch(...)
-		{  }
-		
-		try
-		{
-			Time thistime=time;
-			for(int i=0; i<onions[1]; i++)
-			{
-				Time keytime=get_canvas()->keyframe_list().find_next(thistime)->get_time();
-				onion_skin_queue.push_back(keytime);
-				thistime=keytime;
-			}
-		}
-		catch(...)
-		{  }
-		
-		onion_layers=onion_skin_queue.size();
-		
-		onion_first_tile=false;
-	}
-public:
-	
-	WorkAreaTarget_Cairo_Tile(WorkArea *workarea,int w, int h):
-		workarea(workarea),
-		low_res(workarea->get_low_resolution_flag()),
-		w(w),
-		h(h),
-		real_tile_w(workarea->tile_w),
-		real_tile_h(workarea->tile_h),
-		refresh_id(workarea->refreshes),
-		onionskin(false),
-		onion_first_tile(),
-		onion_layers(0)
-	{
-		set_clipping(true);
-		if(low_res)
-		{
-			int div = workarea->get_low_res_pixel_size();
-			set_tile_w(workarea->tile_w/div);
-			set_tile_h(workarea->tile_h/div);
-		}
-		else
-		{
-			set_tile_w(workarea->tile_w);
-			set_tile_h(workarea->tile_h);
-		}
-		set_canvas(workarea->get_canvas());
-		set_quality(workarea->get_quality());
-	}
-	
-	~WorkAreaTarget_Cairo_Tile()
-	{
-		workarea->queue_draw();
-	}
-	
-	virtual bool set_rend_desc(synfig::RendDesc *newdesc)
-	{
-		assert(workarea);
-		newdesc->set_flags(RendDesc::PX_ASPECT|RendDesc::IM_SPAN);
-		if(low_res) {
-			int div = workarea->get_low_res_pixel_size();
-			newdesc->set_wh(w/div,h/div);
-		}
-		else
-			newdesc->set_wh(w,h);
-		
-		if(
-		   workarea->get_w()!=w
-		   || 	workarea->get_h()!=h
-		   ) workarea->set_wh(w,h,4);
-				
-		desc=*newdesc;
-		workarea->full_frame=false;
-		return true;
-	}
-	
-	virtual int total_tiles()const
-	{
-		int tw(rend_desc().get_w()/get_tile_w());
-		int th(rend_desc().get_h()/get_tile_h());
-		if(rend_desc().get_w()%get_tile_w()!=0)tw++;
-		if(rend_desc().get_h()%get_tile_h()!=0)th++;
-		return tw*th;
-	}
-	
-	virtual int next_frame(Time& time)
-	{
-		synfig::Mutex::Lock lock(mutex);
-		
-		if(!onionskin)
-			return synfig::Target_Cairo_Tile::next_frame(time);
-		
-		onion_first_tile=(onion_layers==(signed)onion_skin_queue.size());
-		
-		if(!onion_skin_queue.empty())
-		{
-			time=onion_skin_queue.front();
-			onion_skin_queue.pop_front();
-		}
-		else
-			return 0;
-		
-		return onion_skin_queue.size()+1;
-	}
-	
-	virtual int next_tile(int& x, int& y)
-	{
-		synfig::Mutex::Lock lock(mutex);
-
-		int curr_tile(workarea->next_unrendered_tile(refresh_id-onion_skin_queue.size()));
-		if(curr_tile<0)
-			return 0;
-		
-		// Width of the image(in tiles)
-		int tw(rend_desc().get_w()/get_tile_w());
-		if(rend_desc().get_w()%get_tile_w()!=0)tw++;
-		
-		y=(curr_tile/tw)*get_tile_w();
-		x=(curr_tile%tw)*get_tile_h();
-		
-		// Mark this tile as "up-to-date"
-		if(onionskin)
-			workarea->cairo_book[curr_tile].refreshes=refresh_id-onion_skin_queue.size();
-		else
-			workarea->cairo_book[curr_tile].refreshes=refresh_id;
-		
-		return total_tiles()-curr_tile+1;
-	}
-	
-	
-	virtual bool start_frame(synfig::ProgressCallback */*cb*/)
-	{
-		synfig::Mutex::Lock lock(mutex);
-		workarea->cairo_book.resize(total_tiles());
-		return true;
-	}
-		
-	virtual bool add_tile(cairo_surface_t* tile_surface, int x, int y)
-	{
-		synfig::Mutex::Lock lock(mutex);
-		if(cairo_surface_status(tile_surface))
-			return false;
-
-		gamma_filter(tile_surface, App::gamma);
-		x/=get_tile_w();
-		y/=get_tile_h();
-		int tw(rend_desc().get_w()/get_tile_w());
-		if(rend_desc().get_w()%get_tile_w()!=0)tw++;
-		unsigned int index=y*tw+x;
-		
-		// Sanity check
-		if(index>workarea->cairo_book.size())
-			return false;
-		
-		if(!onionskin || onion_first_tile || !workarea->cairo_book[index].surface)
-		{
-			if(workarea->cairo_book[index].surface)
-				cairo_surface_destroy(workarea->cairo_book[index].surface);
-			workarea->cairo_book[index].surface=cairo_surface_reference(tile_surface);
-		}
-		else
-		{
-			cairo_t* cr=cairo_create(workarea->cairo_book[index].surface);
-			cairo_set_source_surface(cr, tile_surface, 0, 0);
-			cairo_paint_with_alpha(cr, 1.0/(onion_layers-onion_skin_queue.size()+1));
-			cairo_destroy(cr);
-		}
-		
-		workarea->queue_draw();
-		assert(workarea->cairo_book[index].surface);
-
-		cairo_surface_destroy(tile_surface);
-		return true;
-	}
-	virtual void end_frame()
-	{
-	}
-};
-
-
 class studio::WorkAreaTarget : public synfig::Target_Tile
 {
 public:
@@ -325,6 +105,8 @@ public:
 	bool low_res;
 	int w,h;
 	int real_tile_w,real_tile_h;
+	int max_tile_w,max_tile_h;
+	bool force_fullframe;
 
 	int refresh_id;
 
@@ -334,6 +116,8 @@ public:
 
 	std::list<synfig::Time> onion_skin_queue;
 
+	std::vector<RectInt> tiles_queue;
+
 	synfig::Mutex mutex;
 
 	void set_onion_skin(bool x, int *onions)
@@ -378,14 +162,17 @@ public:
 	}
 public:
 
-	WorkAreaTarget(WorkArea *workarea,int w, int h):
+	WorkAreaTarget(WorkArea *workarea, int w, int h, int max_tile_w, int max_tile_h, bool force_fullframe):
 		workarea(workarea),
 		low_res(workarea->get_low_resolution_flag()),
 		w(w),
 		h(h),
-		real_tile_w(workarea->tile_w),
-		real_tile_h(workarea->tile_h),
-		refresh_id(workarea->refreshes),
+		real_tile_w(workarea->get_tile_w()),
+		real_tile_h(workarea->get_tile_h()),
+		force_fullframe(force_fullframe),
+		max_tile_w(max_tile_w),
+		max_tile_h(max_tile_h),
+		refresh_id(workarea->get_refreshes()),
 		onionskin(false),
 		onion_first_tile(),
 		onion_layers(0)
@@ -422,32 +209,38 @@ public:
 			newdesc->set_wh(w/div,h/div);
 		}
 		else
-			newdesc->set_wh(w,h);
+			newdesc->set_wh(w, h);
 
-		if(
-			 	workarea->get_w()!=w
-			|| 	workarea->get_h()!=h
-		) workarea->set_wh(w,h,4);
+		if ( workarea->get_w() != w
+		  || workarea->get_h() != h )
+			workarea->set_wh(w, h, 4);
 
 		workarea->full_frame=false;
+		desc = *newdesc;
 
-		desc=*newdesc;
 		return true;
 	}
 
-	virtual int total_tiles()const
-	{
-		int tw(rend_desc().get_w()/get_tile_w());
-		int th(rend_desc().get_h()/get_tile_h());
-		if(rend_desc().get_w()%get_tile_w()!=0)tw++;
-		if(rend_desc().get_h()%get_tile_h()!=0)th++;
-		return tw*th;
-	}
-
 	virtual int next_frame(Time& time)
 	{
 		synfig::Mutex::Lock lock(mutex);
 
+		RectInt window_rect = workarea->get_window_rect(get_tile_w(), get_tile_h());
+
+		if (force_fullframe)
+		{
+			tiles_queue.clear();
+			tiles_queue.push_back(window_rect);
+		}
+		else
+		{
+			workarea->get_tile_book().get_dirty_rects(
+				tiles_queue,
+				refresh_id - onion_skin_queue.size(),
+				window_rect,
+				VectorInt(max_tile_w, max_tile_h) );
+		}
+
 		if(!onionskin)
 			return synfig::Target_Tile::next_frame(time);
 
@@ -464,38 +257,19 @@ public:
 		return onion_skin_queue.size()+1;
 	}
 
-	virtual int next_tile(int& x, int& y)
+	virtual int next_tile(RectInt &rect)
 	{
 		synfig::Mutex::Lock lock(mutex);
-		//if(workarea->tile_queue.empty()) return 0;
-
-		//int curr_tile(workarea->tile_queue.front());
-		//workarea->tile_queue.pop_front();
-		int curr_tile(workarea->next_unrendered_tile(refresh_id-onion_skin_queue.size()));
-		if(curr_tile<0)
+		if (tiles_queue.empty())
 			return 0;
 
-		// Width of the image(in tiles)
-		int tw(rend_desc().get_w()/get_tile_w());
-		if(rend_desc().get_w()%get_tile_w()!=0)tw++;
-
-		y=(curr_tile/tw)*get_tile_w();
-		x=(curr_tile%tw)*get_tile_h();
-
-		// Mark this tile as "up-to-date"
-		if(onionskin)
-			workarea->tile_book[curr_tile].second=refresh_id-onion_skin_queue.size();
-		else
-			workarea->tile_book[curr_tile].second=refresh_id;
-
-		return total_tiles()-curr_tile+1;
+		rect = tiles_queue.back();
+		tiles_queue.pop_back();
+		return (int)tiles_queue.size() + 1;
 	}
 
-
 	virtual bool start_frame(synfig::ProgressCallback */*cb*/)
 	{
-		synfig::Mutex::Lock lock(mutex);
-		workarea->tile_book.resize(total_tiles());
 		return true;
 	}
 
@@ -508,7 +282,7 @@ public:
 
 		PixelFormat pf(PF_RGB|PF_A);
 
-		const int total_bytes(get_tile_w()*get_tile_h()*synfig::channels(pf));
+		const int total_bytes(surface.get_w()*surface.get_h()*synfig::channels(pf));
 
 		unsigned char *buffer((unsigned char*)malloc(total_bytes));
 
@@ -528,16 +302,6 @@ public:
 									   );
 		}
 
-		x/=get_tile_w();
-		y/=get_tile_h();
-		int tw(rend_desc().get_w()/get_tile_w());
-		if(rend_desc().get_w()%get_tile_w()!=0)tw++;
-		unsigned int index=y*tw+x;
-
-		// Sanity check
-		if(index>workarea->tile_book.size())
-			return false;
-
 		Glib::RefPtr<Gdk::Pixbuf> pixbuf;
 
 		pixbuf=Gdk::Pixbuf::create_from_data(
@@ -562,14 +326,17 @@ public:
 			);
 		}
 
-		if(!onionskin || onion_first_tile || !workarea->tile_book[index].first)
+		RectInt rect(x, y, x + pixbuf->get_width(), y + pixbuf->get_height());
+		WorkAreaTile *tile = workarea->get_tile_book().find_tile(refresh_id - onion_skin_queue.size() - 1, rect);
+
+		if(!onionskin || onion_first_tile || !tile || !tile->pixbuf)
 		{
-			workarea->tile_book[index].first=pixbuf;
+			workarea->get_tile_book().add(refresh_id - onion_skin_queue.size(), x, y, pixbuf);
 		}
 		else
 		{
 			pixbuf->composite(
-				workarea->tile_book[index].first, // Dest
+				tile->pixbuf, // Dest
 				0,//int dest_x
 				0,//int dest_y
 				pixbuf->get_width(), // dest width
@@ -581,11 +348,12 @@ public:
 				Gdk::INTERP_NEAREST, // interp
 				255/(onion_layers-onion_skin_queue.size()+1) //int overall_alpha
 			);
+			tile->refresh_id = refresh_id - onion_skin_queue.size();
+			workarea->get_tile_book().sort();
 		}
 
 		//if(index%2)
 			workarea->queue_draw();
-		assert(workarea->tile_book[index].first);
 		return true;
 	}
 
@@ -596,161 +364,6 @@ public:
 };
 
 
-class studio::WorkAreaTarget_Cairo: public synfig::Target_Cairo
-{
-public:
-	WorkArea *workarea;
-	bool low_res;
-	int w,h;
-	int refresh_id;
-	bool onionskin;
-	bool onion_first_tile;
-	int onion_layers;
-
-	std::list<synfig::Time> onion_skin_queue;
-
-	void set_onion_skin(bool status, int *onions)
-	{
-		onionskin=status;
-		
-		Time time(rend_desc().get_time_start());
-		
-		if(!onionskin)
-			return;
-		onion_skin_queue.push_back(time);
-		try
-		{
-			Time thistime=time;
-			for(int i=0; i<onions[0]; i++)
-			{
-				Time keytime=get_canvas()->keyframe_list().find_prev(thistime)->get_time();
-				onion_skin_queue.push_back(keytime);
-				thistime=keytime;
-			}
-		}
-		catch(...)
-		{  }
-		
-		try
-		{
-			Time thistime=time;
-			for(int i=0; i<onions[1]; i++)
-			{
-				Time keytime=get_canvas()->keyframe_list().find_next(thistime)->get_time();
-				onion_skin_queue.push_back(keytime);
-				thistime=keytime;
-			}
-		}
-		catch(...)
-		{  }
-		
-		onion_layers=onion_skin_queue.size();
-		
-		onion_first_tile=false;
-	}
-public:
-	
-	WorkAreaTarget_Cairo(WorkArea *workarea,int w, int h):
-		workarea(workarea),
-		low_res(workarea->get_low_resolution_flag()),
-		w(w),
-		h(h),
-		refresh_id(workarea->refreshes),
-		onionskin(false),
-		onion_first_tile(),
-		onion_layers(0)
-	{
-		set_canvas(workarea->get_canvas());
-		set_quality(workarea->get_quality());
-	}
-
-	~WorkAreaTarget_Cairo()
-	{ }
-
-	virtual bool set_rend_desc(synfig::RendDesc *newdesc)
-	{
-		assert(workarea);
-		newdesc->set_flags(RendDesc::PX_ASPECT|RendDesc::IM_SPAN);
-		if(low_res)
-		{
-			int div = workarea->get_low_res_pixel_size();
-			newdesc->set_wh(w/div,h/div);
-		}
-		else
-			newdesc->set_wh(w,h);
-		
-		if(
-		   workarea->get_w()!=w
-		   || 	workarea->get_h()!=h
-		   ) workarea->set_wh(w,h,4);
-				
-		desc=*newdesc;
-		workarea->full_frame=true;
-		workarea->cairo_book.resize(1);
-		return true;
-	}
-
-	virtual int next_frame(Time& time)
-	{
-		// Mark this tile as "up-to-date"
-		if(onionskin)
-			workarea->cairo_book[0].refreshes=refresh_id-onion_skin_queue.size();
-		else
-			workarea->cairo_book[0].refreshes=refresh_id;
-		
-		if(!onionskin)
-			return synfig::Target_Cairo::next_frame(time);
-		
-		onion_first_tile=(onion_layers==(signed)onion_skin_queue.size());
-		
-		if(!onion_skin_queue.empty())
-		{
-			time=onion_skin_queue.front();
-			onion_skin_queue.pop_front();
-		}
-		else
-			return 0;
-		return onion_skin_queue.size()+1;
-	}
-
-	virtual bool obtain_surface(cairo_surface_t*& surface)
-	{
-		int localw=desc.get_w(), localh=desc.get_h();
-		surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, localw, localh);
-		return true;
-	}
-
-	bool put_surface(cairo_surface_t *surf, synfig::ProgressCallback *cb)
-	{
-		if(!workarea)
-			return false;
-		gamma_filter(surf, App::gamma);
-		if(cairo_surface_status(surf))
-		{
-			if(cb) cb->error(_("Cairo Surface bad status"));
-			return false;
-		}
-		
-		if(!onionskin || onion_first_tile || !workarea->cairo_book[0].surface)
-		{
-			workarea->cairo_book[0].surface=cairo_surface_reference(surf);
-		}
-		else
-		{
-			cairo_t* cr=cairo_create(workarea->cairo_book[0].surface);
-			cairo_set_source_surface(cr, surf, 0, 0);
-			cairo_paint_with_alpha(cr, 255/(onion_layers-onion_skin_queue.size()+1));
-		}
-		
-		workarea->queue_draw();
-		assert(workarea->cairo_book[0].surface);
-		
-		cairo_surface_destroy(surf);
-		return true;
-	}
-
-};
-
 class studio::WorkAreaTarget_Full : public synfig::Target_Scanline
 {
 public:
@@ -852,18 +465,11 @@ public:
 
 		desc=*newdesc;
 		workarea->full_frame=true;
-		workarea->tile_book.resize(1);
 		return true;
 	}
 
 	virtual int next_frame(Time& time)
 	{
-		// Mark this tile as "up-to-date"
-		if(onionskin)
-			workarea->tile_book[0].second=refresh_id-onion_skin_queue.size();
-		else
-			workarea->tile_book[0].second=refresh_id;
-
 		if(!onionskin)
 			return synfig::Target_Scanline::next_frame(time);
 
@@ -948,16 +554,17 @@ public:
 			);
 		}
 
-		int index=0;
+		RectInt rect(0, 0, pixbuf->get_width(), pixbuf->get_height());
+		WorkAreaTile *tile = workarea->get_tile_book().find_tile(refresh_id - onion_skin_queue.size() - 1, rect);
 
-		if(!onionskin || onion_first_tile || !workarea->tile_book[index].first)
+		if(!onionskin || onion_first_tile || !tile || !tile->pixbuf)
 		{
-			workarea->tile_book[index].first=pixbuf;
+			workarea->get_tile_book().add(refresh_id - onion_skin_queue.size(), 0, 0, pixbuf);
 		}
 		else
 		{
 			pixbuf->composite(
-				workarea->tile_book[index].first, // Dest
+				tile->pixbuf, // Dest
 				0,//int dest_x
 				0,//int dest_y
 				pixbuf->get_width(), // dest width
@@ -969,18 +576,139 @@ public:
 				Gdk::INTERP_NEAREST, // interp
 				255/(onion_layers-onion_skin_queue.size()+1) //int overall_alpha
 			);
+			tile->refresh_id = refresh_id - onion_skin_queue.size();
+			workarea->get_tile_book().sort();
 		}
 
 		workarea->queue_draw();
-		assert(workarea->tile_book[index].first);
 	}
 };
 
-
-
-
 /* === M E T H O D S ======================================================= */
 
+WorkAreaTile*
+WorkAreaTileBook::find_tile(int refresh_id, const synfig::RectInt &rect)
+{
+	for(WorkAreaTile::List::iterator i = tiles.begin(); i != tiles.end(); ++i)
+		if (i->refresh_id >= refresh_id && i->rect == rect)
+			return &*i;
+	return NULL;
+}
+
+void
+WorkAreaTileBook::sort()
+{
+	tiles.sort();
+	for(WorkAreaTile::List::iterator i = tiles.begin(); i != tiles.end(); ++i)
+		for(WorkAreaTile::List::iterator j = tiles.begin(); j != i;)
+			if (j->refresh_id < i->refresh_id && etl::intersect(i->rect, j->rect))
+				{ WorkAreaTile::List::iterator k = j; ++j; tiles.erase(k); } else ++j;
+}
+
+void
+WorkAreaTileBook::add(const WorkAreaTile &tile)
+{
+	if (!tile.rect.valid() || !tile.pixbuf)
+		return;
+	tiles.push_back(tile);
+	sort();
+}
+
+void
+WorkAreaTileBook::get_dirty_rects(
+	std::vector<synfig::RectInt> &out_rects,
+	int refresh_id,
+	const synfig::RectInt &bounds,
+	const synfig::VectorInt &max_size) const
+{
+	out_rects.clear();
+	if (!bounds.valid())
+		return;
+
+	out_rects.push_back(bounds);
+	std::vector<synfig::RectInt> subs;
+	subs.reserve(4);
+
+	// remove updated rects
+	for(WorkAreaTile::List::const_iterator i = tiles.begin(); i != tiles.end(); ++i)
+	{
+		assert(i->rect.valid());
+		if (i->refresh_id >= refresh_id)
+		{
+			for(std::vector<synfig::RectInt>::iterator j = out_rects.begin(); j != out_rects.end();)
+			{
+				assert(j->valid());
+				RectInt rect = *j;
+				etl::set_intersect(rect, rect, i->rect);
+				if (rect.valid())
+				{
+					subs.clear();
+					if (rect.minx != j->minx)
+						subs.push_back(RectInt(j->minx, j->miny, rect.minx, j->maxy));
+					if (rect.maxx != j->maxx)
+						subs.push_back(RectInt(rect.maxx, j->miny, j->maxx, j->maxy));
+					if (rect.miny != j->miny)
+						subs.push_back(RectInt(rect.minx, j->miny, rect.maxx, rect.miny));
+					if (rect.maxy != j->maxy)
+						subs.push_back(RectInt(rect.minx, rect.maxy, rect.maxx, j->maxy));
+
+					if (subs.empty())
+						j = out_rects.erase(j);
+					else
+					{
+						*j = subs.front();
+						int index = j - out_rects.begin();
+						out_rects.insert(j+1, subs.begin()+1, subs.end());
+						j = out_rects.begin() + index + subs.size();
+					}
+				}
+				else
+					++j;
+			}
+		}
+	}
+
+	// merge rects
+	for(int i = 0; i < (int)out_rects.size(); ++i)
+	{
+		RectInt &r = out_rects[i];
+		for(std::vector<synfig::RectInt>::iterator j = out_rects.begin(); j != out_rects.end();)
+		{
+			bool remove = false;
+			if (r.minx == j->minx && r.maxx == j->maxx && r.maxy == j->miny)
+				{ r.maxy = j->maxy; remove = true; }
+			if (r.minx == j->minx && r.maxx == j->maxx && r.miny == j->maxy)
+				{ r.miny = j->miny; remove = true; }
+			if (r.miny == j->miny && r.maxy == j->maxy && r.maxx == j->minx)
+				{ r.maxx = j->maxx; remove = true; }
+			if (r.miny == j->miny && r.maxy == j->maxy && r.minx == j->maxx)
+				{ r.minx = j->minx; remove = true; }
+			if (remove) j = out_rects.erase(j); else ++j;
+		}
+	}
+
+	// split large rects
+	for(std::vector<synfig::RectInt>::iterator j = out_rects.begin(); j != out_rects.end();)
+	{
+		assert(j->valid());
+		if (j->maxx - j->minx > max_size[0])
+		{
+			j = out_rects.insert(j+1, RectInt(j->minx + max_size[0], j->miny, j->maxx, j->maxy));
+			(j-1)->maxx = j->minx;
+			--j;
+		}
+		else
+		if (j->maxy - j->miny > max_size[1])
+		{
+			j = out_rects.insert(j+1, RectInt(j->minx, j->miny + max_size[1], j->maxx, j->maxy));
+			(j-1)->maxy = j->miny;
+			--j;
+		}
+		else
+			++j;
+	}
+}
+
 
 WorkArea::WorkArea(etl::loose_handle<synfigapp::CanvasInterface> canvas_interface):
 	Gtk::Table(3, 3, false), /* 3 columns by 3 rows*/
@@ -1032,17 +760,18 @@ WorkArea::WorkArea(etl::loose_handle<synfigapp::CanvasInterface> canvas_interfac
 
 	meta_data_lock=false;
 
-	insert_renderer(new Renderer_Background,000);
-	insert_renderer(new Renderer_Canvas,	010);
-	insert_renderer(new Renderer_Grid,		100);
-	insert_renderer(new Renderer_Guides,	200);
-	insert_renderer(new Renderer_Ducks,		300);
-	insert_renderer(new Renderer_BBox,		399);
-	insert_renderer(new Renderer_Dragbox,	400);
-	insert_renderer(new Renderer_Timecode,	500);
-	insert_renderer(new Renderer_BoneSetup,	501);
+	insert_renderer(new Renderer_Background, 000);
+	insert_renderer(new Renderer_Canvas,     010);
+	insert_renderer(new Renderer_Grid,       100);
+	insert_renderer(new Renderer_Guides,     200);
+	insert_renderer(new Renderer_Ducks,      300);
+	insert_renderer(new Renderer_BBox,       399);
+	insert_renderer(new Renderer_Dragbox,    400);
+	insert_renderer(new Renderer_Timecode,   500);
+	insert_renderer(new Renderer_BoneSetup,  501);
 
 	signal_duck_selection_changed().connect(sigc::mem_fun(*this,&studio::WorkArea::queue_draw));
+	signal_duck_selection_single().connect(sigc::mem_fun(*this, &studio::WorkArea::on_duck_selection_single));
 	signal_strokes_changed().connect(sigc::mem_fun(*this,&studio::WorkArea::queue_draw));
 	signal_grid_changed().connect(sigc::mem_fun(*this,&studio::WorkArea::queue_draw));
 	signal_grid_changed().connect(sigc::mem_fun(*this,&studio::WorkArea::save_meta_data));
@@ -1108,11 +837,12 @@ WorkArea::WorkArea(etl::loose_handle<synfigapp::CanvasInterface> canvas_interfac
 	vscrollbar1->show();
 	attach(*vscrollbar1, 2, 3, 1, 2, Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
 
-	ZoomDial *zoomdial=manage(new class ZoomDial(iconsize));
+	zoomdial = manage(new class ZoomDial(iconsize));
 	zoomdial->signal_zoom_in().connect(sigc::mem_fun(*this, &studio::WorkArea::zoom_in));
 	zoomdial->signal_zoom_out().connect(sigc::mem_fun(*this, &studio::WorkArea::zoom_out));
 	zoomdial->signal_zoom_fit().connect(sigc::mem_fun(*this, &studio::WorkArea::zoom_fit));
 	zoomdial->signal_zoom_norm().connect(sigc::mem_fun(*this, &studio::WorkArea::zoom_norm));
+	zoomdial->signal_zoom_edit().connect(sigc::mem_fun(*this, &studio::WorkArea::zoom_edit));
 
 	hbox->pack_end(*hscrollbar1, Gtk::PACK_EXPAND_WIDGET,0);
 	hscrollbar1->show();
@@ -1818,7 +1548,6 @@ WorkArea::set_wh(int W, int H,int CHAN)
 	refresh_dimension_info();
 
 	tile_book.clear();
-	cairo_book.clear();
 
 	return true;
 }
@@ -1939,7 +1668,7 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 		if (gdk_device_get_axis(device, event->motion.axes, GDK_AXIS_PRESSURE, &p))
 			p = std::max(0.0, (p - 0.04)/(1.0 - 0.04)); else p = 1.0;
 
-		if(isnan(x) || isnan(y) || isnan(p))
+		if(std::isnan(x) || std::isnan(y) || std::isnan(p))
 			return false;
 
 		mouse_pos=synfig::Point(screen_to_comp_coords(synfig::Point(x, y)));
@@ -1981,7 +1710,7 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 		double x = 0.0, y = 0.0, p = 0.0;
 		int ox = 0, oy = 0;
 		Gtk::Container *toplevel = drawing_frame->get_toplevel();
-		drawing_frame->translate_coordinates(*toplevel, 0, 0, ox, oy);
+		if (toplevel) drawing_frame->translate_coordinates(*toplevel, 0, 0, ox, oy);
 
 		if (gdk_device_get_axis(device, event->motion.axes, GDK_AXIS_X, &x))
 			x -= ox; else x = event->motion.x;
@@ -1990,7 +1719,7 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 		if (gdk_device_get_axis(device, event->motion.axes, GDK_AXIS_PRESSURE, &p))
 			p = std::max(0.0, (p - 0.04)/(1.0 - 0.04)); else p = 1.0;
 
-		if(isnan(x) || isnan(y) || isnan(p))
+		if(std::isnan(x) || std::isnan(y) || std::isnan(p))
 			return false;
 
 		mouse_pos=synfig::Point(screen_to_comp_coords(synfig::Point(x, y)));
@@ -2087,33 +1816,36 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 
 					// we have the tangent, but need the vertex - that's the parent
 					if (value_desc.is_value_node()) {
-						ValueNode_Composite::Handle value_node = value_desc.get_value_node();
-						BLinePoint bp((*value_node)(get_time()).get(BLinePoint()));
-						// if the tangent isn't split, then split it
-						if (!bp.get_split_tangent_both())
+						if (ValueNode_Composite::Handle value_node = ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node()))
 						{
-							if (get_canvas_view()->canvas_interface()->change_value(synfigapp::ValueDesc(
-									value_node,
-									value_node->get_link_index_from_name("split_radius")),
-									true)
-							 && get_canvas_view()->canvas_interface()->change_value(synfigapp::ValueDesc(
-									value_node,
-									value_node->get_link_index_from_name("split_angle")),
-									true )
-							)
+							BLinePoint bp((*value_node)(get_time()).get(BLinePoint()));
+							// if the tangent isn't split, then split it
+							if (!bp.get_split_tangent_both())
 							{
-								// rebuild the ducks from scratch, so the tangents ducks aren't connected
-								get_canvas_view()->rebuild_ducks();
+								if (get_canvas_view()->canvas_interface()->change_value(synfigapp::ValueDesc(
+										value_node,
+										value_node->get_link_index_from_name("split_radius")),
+										true)
+								 && get_canvas_view()->canvas_interface()->change_value(synfigapp::ValueDesc(
+										value_node,
+										value_node->get_link_index_from_name("split_angle")),
+										true )
+								)
+								{
+									// rebuild the ducks from scratch, so the tangents ducks aren't connected
+									get_canvas_view()->rebuild_ducks();
 
-								// reprocess the mouse click
-								return on_drawing_area_event(event);
+									// reprocess the mouse click
+									return on_drawing_area_event(event);
+								}
+								else
+									return true;
 							}
-							else
-								return true;
+						} else {
+							synfig::info("parent isn't composite value node?");
 						}
 					} else {
 						// I don't know how to access the vertex from the tangent duck when originally drawing the bline in the bline tool
-
 						// synfig::ValueNode::Handle vn = value_desc.get_value_node();
 						synfig::info("parent isn't value node?  shift-drag-tangent doesn't work in bline tool yet...");
 					}
@@ -2390,24 +2122,14 @@ WorkArea::on_drawing_area_event(GdkEvent *event)
 		{
 		case DRAG_GUIDE :
 		{
-			double y,x;
-			if(*(event->button.axes))
-			{
-				x=(event->button.axes[0]);
-				y=(event->button.axes[1]);
-			}
-			else
-			{
-				x=event->button.x;
-				y=event->button.y;
-			}
+			double y(event->button.y),x(event->button.x);
 
 			// Erase the guides if dragged into the rulers
-			if(curr_guide_is_x && !isnan(x) && x<0.0 )
+			if(curr_guide_is_x && !std::isnan(x) && x<0.0 )
 			{
 				get_guide_list_x().erase(curr_guide);
 			}
-			else if(!curr_guide_is_x && !isnan(y) && y<0.0 )
+			else if(!curr_guide_is_x && !std::isnan(y) && y<0.0 )
 			{
 				get_guide_list_y().erase(curr_guide);
 			}
@@ -2704,10 +2426,6 @@ WorkArea::on_hruler_event(GdkEvent *event)
 		{
 			// Event is in the hruler, which has a slightly different
 			// coordinate system from the canvas.
-			gint exes_count = gdk_device_get_n_axes(event->motion.device);
-			for(gint i = 0; i < exes_count; ++i)
-				if (gdk_device_get_axis_use(event->motion.device, i) == GDK_AXIS_Y)
-					event->motion.axes[i] -= hruler->get_height()+2;
 			event->motion.y -= hruler->get_height()+2;
 
 			// call the on drawing area event to refresh eveything.
@@ -2752,10 +2470,6 @@ WorkArea::on_vruler_event(GdkEvent *event)
 		{
 			// Event is in the vruler, which has a slightly different
 			// coordinate system from the canvas.
-			gint exes_count = gdk_device_get_n_axes(event->motion.device);
-			for(gint i = 0; i < exes_count; ++i)
-				if (gdk_device_get_axis_use(event->motion.device, i) == GDK_AXIS_X)
-					event->motion.axes[i] -= vruler->get_width()+2;
 			event->motion.x -= vruler->get_width()+2;
 
 			// call the on drawing area event to refresh eveything.
@@ -2779,6 +2493,15 @@ WorkArea::on_vruler_event(GdkEvent *event)
 	return false;
 }
 
+void
+WorkArea::on_duck_selection_single(const etl::handle<Duck>& duck)
+{
+	if(dragging == DRAG_NONE)
+	{
+		studio::LayerTree* tree_layer(dynamic_cast<studio::LayerTree*>(canvas_view->get_ext_widget("layers_cmp")));
+		tree_layer->select_param(duck->get_value_desc());
+	}
+}
 
 void
 WorkArea::refresh_dimension_info()
@@ -2844,17 +2567,9 @@ WorkArea::comp_to_screen_coords(synfig::Point /*pos*/)const
 	return synfig::Point();
 }
 
-int
-WorkArea::next_unrendered_tile(int refreshes)const
+synfig::VectorInt
+WorkArea::get_windows_offset() const
 {
-	bool uses_cairo=studio::App::workarea_uses_cairo;
-	if(tile_book.empty() && !uses_cairo)
-		return -1;
-	else if (cairo_book.empty() && uses_cairo)
-		return -1;
-
-	//const synfig::RendDesc &rend_desc(get_canvas()->rend_desc());
-
 	const synfig::Vector focus_point(get_focus_point());
 
 	// Calculate the window coordinates of the top-left
@@ -2863,86 +2578,29 @@ WorkArea::next_unrendered_tile(int refreshes)const
 		x(focus_point[0]/pw+drawing_area->get_width()/2-w/2),
 		y(focus_point[1]/ph+drawing_area->get_height()/2-h/2);
 
-	int div = low_res_pixel_size;
-	const int width_in_tiles(w/tile_w+((low_resolution?((w/div)%(tile_w/div)):(w%tile_w))?1:0));
-	const int height_in_tiles(h/tile_h+(h%tile_h?1:0));
-
-	int
-		u(0),v(0),
-		u1(int(-x/tile_w)),
-		v1(int(-y/tile_h)),
-		u2(int((-x+drawing_area->get_width())/tile_w+1)),
-		v2(int((-y+drawing_area->get_height())/tile_h+1));
-
-	if(u2>width_in_tiles)u2=width_in_tiles;
-	if(v2>height_in_tiles)v2=height_in_tiles;
-	if(u1<0)u1=0;
-	if(v1<0)v1=0;
-
-	int last_good_tile(-1);
-
-	for(v=v1;v<v2;v++)
-		for(u=u1;u<u2;u++)
-		{
-			int index(v*width_in_tiles+u);
-			if(!uses_cairo)
-			{
-				if(tile_book[index].second<refreshes)
-				{
-					last_good_tile=index;
-					if(rand()%8==0)
-						return index;
-				}
-			}
-			else
-			{
-				
-				if(cairo_book[index].refreshes<refreshes)
-				{
-					last_good_tile=index;
-					if(rand()%8==0)
-						return index;
-				}
-			}
-			
-		}
-	return last_good_tile;
+	return VectorInt(int(x), int(y));
 }
 
-/*
-template <typename F, typename T=WorkAreaRenderer, typename R=typename F::result_type>
-class handle2ptr_t : public std::unary_function<typename etl::handle<T>,R>
+synfig::RectInt
+WorkArea::get_window_rect(int stepx, int stepy) const
 {
-private:
-	F func;
-public:
-	handle2ptr_t(const F &func):func(func) { };
+	VectorInt offset = get_windows_offset();
 
-	R operator()(typename etl::handle<T> x) { return func(*x); }
-};
+	RectInt rect(
+		-offset[0],
+		-offset[1],
+		-offset[0] + drawing_area->get_width(),
+		-offset[1] + drawing_area->get_height() );
 
-template <typename F>
-handle2ptr_t<F>
-handle2ptr(F func)
-{
-	return handle2ptr_t<F>(func);
+	rect.minx = rect.minx < 0 ? ((rect.minx+1)/stepx-1)*stepx : rect.minx/stepx*stepx;
+	rect.maxx = rect.maxx < 0 ? rect.maxx/stepx*stepx : ((rect.maxx-1)/stepx+1)*stepx;
+	rect.miny = rect.miny < 0 ? ((rect.miny+1)/stepy-1)*stepy : rect.miny/stepy*stepy;
+	rect.maxy = rect.maxy < 0 ? rect.maxy/stepy*stepy : ((rect.maxy-1)/stepy+1)*stepy;
+
+	etl::set_intersect(rect, rect, RectInt(0, 0, w, h));
+	return rect;
 }
-	for_each(
-		renderer_set_.begin(),
-		renderer_set_.end(),
-		handle2ptr(
-			sigc::bind(
-				sigc::bind(
-					sigc::mem_fun(
-						&WorkAreaRenderer::render_vfunc
-					),
-					Gdk::Rectangle(event->area)
-				),
-				drawing_area->get_window()
-			)
-		)
-	);
-*/
+
 
 #ifdef SINGLE_THREADED
 /* resize bug workaround */
@@ -3005,7 +2663,7 @@ WorkArea::refresh(const Cairo::RefPtr<Cairo::Context> &cr)
 		}
 	}
 #endif
-	
+
 	assert(get_canvas());
 
 	//!Check if the window we want draw is ready
@@ -3224,42 +2882,15 @@ studio::WorkArea::async_update_preview()
 	// if we have lots of pixels to render and the tile renderer isn't disabled, use it
 	int div;
 	div = low_resolution ? low_res_pixel_size : 1;
-	if(studio::App::workarea_uses_cairo)
-	{
-		if ((w*h > 240*div*135*div && !getenv("SYNFIG_DISABLE_TILE_RENDER")) || getenv("SYNFIG_FORCE_TILE_RENDER"))
-		{
-			handle<WorkAreaTarget_Cairo_Tile> trgt(new class WorkAreaTarget_Cairo_Tile(this,w,h));
-			trgt->set_rend_desc(&desc);
-			trgt->set_onion_skin(get_onion_skin(), onion_skins);
-			target=trgt;
-		}
-		else
-		{
-			handle<WorkAreaTarget_Cairo> trgt(new class WorkAreaTarget_Cairo(this,w,h));
-			trgt->set_rend_desc(&desc);
-			trgt->set_onion_skin(get_onion_skin(), onion_skins);
-			target=trgt;
-		}
 
-	}
-	else if ((w*h > 240*div*135*div && !getenv("SYNFIG_DISABLE_TILE_RENDER")) || getenv("SYNFIG_FORCE_TILE_RENDER"))
-	{
-		// do a tile render
-		handle<WorkAreaTarget> trgt(new class WorkAreaTarget(this,w,h));
+	// do a tile render
+	handle<WorkAreaTarget> trgt(new class WorkAreaTarget(this,w,h,2048,2048,false));
 
-		trgt->set_rend_desc(&desc);
-		trgt->set_onion_skin(get_onion_skin(), onion_skins);
-		target=trgt;
-	}
-	else
-	{
-		// do a scanline render
-		handle<WorkAreaTarget_Full> trgt(new class WorkAreaTarget_Full(this,w,h));
-
-		trgt->set_rend_desc(&desc);
-		trgt->set_onion_skin(get_onion_skin(), onion_skins);
-		target=trgt;
-	}
+	trgt->set_rend_desc(&desc);
+	trgt->set_onion_skin(get_onion_skin(), onion_skins);
+	//trgt->set_allow_multithreading(true);
+	trgt->set_engine(App::workarea_renderer);
+	target=trgt;
 
 	// We can rest assured that our time has already
 	// been set, so there is no need to have to
@@ -3273,13 +2904,10 @@ studio::WorkArea::async_update_preview()
 	);
 
 	rendering=true;
-	async_renderer->start();
-
 	synfig::ProgressCallback *cb=get_canvas_view()->get_ui_interface().get();
-
-	rendering=true;
 	cb->task(_("Rendering..."));
-	rendering=true;
+
+	async_renderer->start();
 
 	return true;
 }
@@ -3301,10 +2929,10 @@ studio::WorkArea::async_update_finished()
 		Real execution_time = async_renderer->get_execution_time();
 		if (execution_time > 0.0)
 		{
-			cb->task( strprintf("%s (%s %f %s)",
-				_("Idle"),
-				_("Last rendering time"),
+			cb->task( strprintf("%s %f (%f) %s",
+				_("Rendered:"),
 				async_renderer->get_execution_time(),
+				async_renderer->get_execution_clock(),
 				_("sec") ));
 		}
 		else
@@ -3380,13 +3008,10 @@ again:
 	set_rend_desc(desc);
 
 	// Create the render target
-	handle<Target> target;
-	if(studio::App::workarea_uses_cairo)
-		target=new class WorkAreaTarget_Cairo(this,w,h);
-	else
-		target=new class WorkAreaTarget(this, w, h);
-
+	handle<WorkAreaTarget> target = new WorkAreaTarget(this,w,h,2048,2048,true);
 	target->set_rend_desc(&desc);
+	//target->set_allow_multithreading(false);
+	target->set_engine(App::workarea_renderer);
 
 	// We can rest assured that our time has already
 	// been set, so there is no need to have to
@@ -3586,6 +3211,12 @@ studio::WorkArea::zoom_norm()
 	set_zoom(1.0f);
 }
 
+void
+studio::WorkArea::zoom_edit()
+{
+	set_zoom(zoomdial->get_zoom(zoom));
+}
+
 gboolean
 studio::WorkArea::__render_preview(gpointer data)
 {
@@ -3676,7 +3307,7 @@ studio::WorkArea::set_cursor(const Glib::RefPtr<Gdk::Cursor> &x)
 	//!Check if the window we want draw is ready
 	Glib::RefPtr<Gdk::Window> draw_area_window = drawing_area->get_window();
 	if(!draw_area_window) return;
-	
+
 	draw_area_window->set_cursor(x);
 }
 void
@@ -3685,7 +3316,7 @@ studio::WorkArea::set_cursor(Gdk::CursorType x)
 	//!Check if the window we want draw is ready
 	Glib::RefPtr<Gdk::Window> draw_area_window = drawing_area->get_window();
 	if(!draw_area_window) return;
-	
+
 	draw_area_window->set_cursor(Gdk::Cursor::create(x));
 }
 
@@ -3702,7 +3333,7 @@ studio::WorkArea::reset_cursor()
 	//!Check if the window we want draw is ready
 	Glib::RefPtr<Gdk::Window> draw_area_window = drawing_area->get_window();
 	if(!draw_area_window) return;
-	
+
 	draw_area_window->set_cursor(Gdk::Cursor::create(Gdk::TOP_LEFT_ARROW));
 //	set_cursor(Gdk::TOP_LEFT_ARROW);
 }
@@ -3711,9 +3342,11 @@ void
 studio::WorkArea::set_zoom(float z)
 {
 	z=max(1.0f/128.0f,min(128.0f,z));
+	zoomdial->set_zoom(z);
 	if(z==zoom)
 		return;
 	zoom = z;
+
 	refresh_dimension_info();
 	/*if(async_renderer)
 	{
