@@ -36,6 +36,8 @@
 #include "valuenodelinkconnect.h"
 */
 
+#include <synfig/general.h>
+
 #include "valuedescconnect.h"
 
 #include "valuedescconvert.h"
@@ -43,7 +45,7 @@
 #include <synfigapp/canvasinterface.h>
 #include <synfig/valuenodes/valuenode_const.h>
 
-#include <synfigapp/general.h>
+#include <synfigapp/localization.h>
 
 #endif
 
